## outofSampleGeneration.py
## input: 
# N0 -- number of scenarios
# number of surgeries in every scenarios
# mean of surgery durations
# std of surgery durations
# 
## output:
# a .txt file containing N0 scenarios for out-of-sample use of surgery durations following lognormal distribution of given mean and std

import math
import numpy



def main():

	## input
	N0 = 10000

	numSurgery = 50
	testGroup = 14
	testNum = {1:6, 2:9, 3:1, 4:2, 5:6, 6:1, 7:5, 8:8, 9:3, 10:1, 11:3, 12:1, 13:2, 14:2}
	surgeryMean = {1:33.0, 2:36.0, 3:19.78, 4:20.49, 5:20.93, 6:40.5, 7:34.1, 8:41.63, 9:77.66, 10:53.3, 11:31.3, 12:138.16, 13:55.78, 14:80.33}
	surgeryStd = {1:19.11, 2:33.88, 3:12.12, 4:10.86, 5:15.08, 6:26.12, 7:17.42, 8:16.43, 9:44.03, 10:27.7, 11:16.37, 12:56.77, 13:22.89, 14:43.76}
	
## data geneartion

	observXi = {} # dictionary of observations

	numpy.random.seed(1) # initialize internal state of the random number generator. can be modified to any other integer.
	
	for iteration in range(1,N0+1):
		for i in range(1, testGroup+1):
			mu = math.log(surgeryMean[i]/math.sqrt((surgeryStd[i]/surgeryMean[i])**2 + 1.0)) # calculate the mean of the underlying normal dist.
			sigma = math.sqrt(math.log((surgeryStd[i]/surgeryMean[i])**2 + 1.0)) # calculate the std of the underlying normal dist.
			observXi[iteration,i] = numpy.random.lognormal(mu, sigma, testNum[i]) # generate lognormal distributed surgery durations
	

## output observXi to .txt file
	saveFile = 'outofSample_'+str(N0)+'.txt'
	outputFile = open(saveFile, 'w')
	outputFile.write('N0 = %s, number of surgeries = %s\n\n'%(N0, numSurgery))
	outputFile.write('############ Out-of-sample Scenarios ############\n\n')
	for iteration in range(1,N0+1):
		outputFile.write('Scenario %s:\t'%(iteration))
		for i in range(1, testGroup):
			for n in range(0, testNum[i]):
				outputFile.write('%s,\t'%(observXi[iteration, i][n]))
		for n in range(0, testNum[testGroup]-1):
			outputFile.write('%s,\t'%(observXi[iteration, testGroup][n]))
		outputFile.write('%s\n\n'%(observXi[iteration, testGroup][testNum[testGroup]-1]))
	outputFile.close()

	

			
	
		

if __name__ == '__main__':
	main()