## Parameters.py
## input: none
## output: To generate parameters for ccsp/drccsp

from gurobipy import *
import math
import numpy


def main():
	
	numOR = 8 # number of ORs
	costOR = {1:0.9, 2:0.9, 3:0.9, 4:1.0, 5:1.0, 6:1.0, 7:1.1, 8:1.1}	# cost of openning each OR
	timeOR = {1:8, 2:8, 3:8, 4:9, 5:9, 6:9, 7:10, 8:10}		# operating time limit of each OR
	
	beta = 0.9 # 0.85~1.0 # the probability bound of all assigned surgeries being completed within OR's time limit 
	alpha = 1.0 # the probability bound that the atual finish time of a surgery cannot be longer than the planned start time of the next surgery
	d = 1.7 # a constant

	v_x2_beta = beta + (math.sqrt(d**2+4*d*(beta-beta**2)) - (2*beta-1)*d)/(2*d+2)
	v_x2_alpha = alpha + (math.sqrt(d**2+4*d*(alpha-alpha**2)) - (2*alpha-1)*d)/(2*d+2)

	v_KL_beta = 0.999
	v_KL_alpha = 1.0


	saveFile = 'Parameters.txt'
	outputFile = open(saveFile, 'w')
	outputFile.write('numOR = %s\n'%(numOR))
	outputFile.write('c_j: %s\n'%(costOR.values()))
	outputFile.write('T_j (hours): %s\n'%(timeOR.values()))
	outputFile.write('d = %s\n'%(d))
	outputFile.write('alpha = %s, beta = %s\n'%(alpha,beta))
	outputFile.write('V_x2(alpha,d) = %s, V_x2(beta,d) = %s\n'%(v_x2_alpha,v_x2_beta))
	outputFile.write('V_KL(alpha,d) = %s, V_KL(beta,d) = %s\n'%(v_KL_alpha,v_KL_beta))
	outputFile.close()
	
if __name__ == '__main__':
	main()