## sampleGenerationCCSP.py
## input: 
# N -- list of scenario numbers
# N0 observation senarios
# 
## output:
# .txt files containing N scenarios of surgery durations following Normal, Lognormal, Gamma distributions with empirical mean and std from the N0 observations

import math
import numpy
import re

N = [100] # 200,300,400,500


# read N0 observations
def readObserv(fileName):

	observ = {}
	numObserv = 0
	with open(fileName) as file:
		for line in file:
			if line.startswith('Scenario'):
				numObserv = numObserv + 1
				m = re.search("(Scenario \d+:\t)", line)
				elements = re.split(",\t", line[len(m.group(1)):])
				eleCounter = 0
				for e in elements:
					eleCounter = eleCounter + 1
					observ[numObserv,eleCounter] = float(e)
	
	return observ, eleCounter, numObserv

# write scenarios into text file
def saveText(fileName, N, scenario, numSurgery):
	for t in N:
		outputFile = open(str(t) + fileName, 'w')
		outputFile.write('N = %s, number of surgeries = %s\n\n'%(t, numSurgery))
		outputFile.write('############ Surgery duration observations ############\n\n')
		for iteration in range(1,t+1):
			outputFile.write('Scenario %s:\t'%(iteration))
			for i in range(1,numSurgery):
				outputFile.write('%s,\t'%(scenario[t,i][iteration-1]))
			outputFile.write('%s\n\n'%(scenario[t,numSurgery][iteration-1]))
		outputFile.close()

def main():

	observFile = 'observations_50.txt'
	observInst, numSurgery, N0 = readObserv(observFile)

	empiricalMean = {}
	empiricalStd = {}
	for i in range(1,numSurgery+1):
		empiricalMean[i] =  sum(observInst[n,i] for n in range(1,N0+1))/float(N0)
		empiricalStd[i] = math.sqrt(sum((observInst[n,i] - empiricalMean[i])**2 for n in range(1,N0+1))/(float(N0-1)))

## generate scenarios for solving ccsp
	scenarioNormal = {} # scearios following Normal dist
	scenarioLogNormal = {} # scearios following LogNormal dist
	scenarioGamma = {} # scearios following Gamma dist
	
	numpy.random.seed(0)  # initialize internal state of the random number generator. can be modified to any other integer.
	for t in N:
		for i in range(1,numSurgery+1):
			scenarioNormal[t,i] = numpy.random.normal(empiricalMean[i], empiricalStd[i], t)

			LogNormalMean = math.log(empiricalMean[i]/math.sqrt((empiricalStd[i]/empiricalMean[i])**2 + 1.0))
			LogNormalStd = math.sqrt(math.log((empiricalStd[i]/empiricalMean[i])**2 + 1.0))
			scenarioLogNormal[t,i] = numpy.random.lognormal(LogNormalMean,LogNormalStd, t)

			theta = empiricalStd[i]**2/empiricalMean[i]
			k = empiricalMean[i]**2/(empiricalStd[i]**2)
			scenarioGamma[t,i] = numpy.random.gamma(k,theta, t)

## write into text files
	saveFile = 'scenariosCCSP.txt'
	saveText('_Normal_'+saveFile, N, scenarioNormal, numSurgery)
	saveText('_LogNormal_'+saveFile, N, scenarioLogNormal, numSurgery)
	saveText('_Gamma_'+saveFile, N, scenarioGamma, numSurgery)

if __name__ == '__main__':
	main()