## sampleGenerationDRCCSP.py
## input: 
# N -- list of scenario numbers
# N0 observation senarios
# 
## output:
# .txt files containing N scenarios of surgery durations following the empirical distribution with empirical support info from the N0 observations

import math
import numpy
import re

N = [500] 


# read N0 observations
def readObserv(fileName):

	observ = {}
	numObserv = 0
	with open(fileName) as file:
		for line in file:
			if line.startswith('Scenario'):
				numObserv = numObserv + 1
				m = re.search("(Scenario \d+:\t)", line)
				elements = re.split(",\t", line[len(m.group(1)):])
				eleCounter = 0
				for e in elements:
					eleCounter = eleCounter + 1
					observ[numObserv,eleCounter] = float(e)
	
	return observ, eleCounter, numObserv

# write scenarios into text file
def saveText(fileName, N, scenario, numSurgery):
	for t in N:
		outputFile = open(str(t) + fileName, 'w')
		outputFile.write('N = %s, number of surgeries = %s\n\n'%(t, numSurgery))
		outputFile.write('############ Surgery duration observations ############\n\n')
		for iteration in range(1,t+1):
			outputFile.write('Scenario %s:\t'%(iteration))
			for i in range(1,numSurgery):
				outputFile.write('%s,\t'%(scenario[t,i][iteration-1]))
			outputFile.write('%s\n\n'%(scenario[t,numSurgery][iteration-1]))
		outputFile.close()


def main():
	interval = 10.0 # interval length for building the empirical distribution

	observFile = 'observations_50.txt'
	observInst, numSurgery, N0 = readObserv(observFile)

	empiricalMin = {}
	empiricalMax = {}
	empiricalSum = {}
	for i in range(1,numSurgery+1):
		empiricalMin[i] = min(observInst[n,i] for n in range(1,N0+1))
		empiricalMax[i] = max(observInst[n,i] for n in range(1,N0+1))
		# empiricalSum[i] = sum(observInst[n,i] for n in range(1,N0+1))

	empiricalDist = {}
	empiricalCD = {}

	numInterval = {}
	for i in range(1,numSurgery+1):
		numInterval[i] = int((empiricalMax[i] - empiricalMin[i])/interval)+1
		empiricalCD[i,0] = 0.0
		for t in range(1,numInterval[i]+1):
			empiricalDist[i,t] = sum(1.0 for n in range(1,N0+1) if observInst[n,i] < empiricalMin[i]+t*interval and observInst[n,i] >= empiricalMin[i]+t*interval-interval-1e-5)/float(N0)
			empiricalCD[i,t] = empiricalCD[i,t-1] + empiricalDist[i,t] 

	scenariosDR = {}
	numpy.random.seed(0)
	for j in N:
		for i in range(1,numSurgery+1):
			tmpRand = numpy.random.rand(j)
			scenariosDR[j,i] = numpy.zeros(j)
			for n in range(1,j+1):
				for t in range(1,numInterval[i]+1):
					if tmpRand[n-1] < empiricalCD[i,t] and tmpRand[n-1] >= empiricalCD[i,t-1]:
						scenariosDR[j,i][n-1] = empiricalMin[i]+t*interval-interval/2.0
		
	# write into text files
	saveFile = '_scenariosDRCCSP.txt'
	saveText(saveFile, N, scenariosDR, numSurgery)
	
if __name__ == '__main__':
	main()